@@ -2,6 +2,7 @@
 
 ;; Copyright © 2014 Chris Done. All rights reserved.
 ;;             2016 Arthur Fayzrakhmanov
+;;             2020 Marc Berkowitz <mberkowitz@github.com>
 
 ;; This file is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -245,9 +246,6 @@ list of modules where missed IDENT was found."
                (haskell-process-suggest-imports session file modules ident)))
            (when haskell-process-suggest-haskell-docs-imports
              (let ((modules (haskell-process-haskell-docs-ident ident)))
-               (haskell-process-suggest-imports session file modules ident)))
-           (when haskell-process-suggest-hayoo-imports
-             (let ((modules (haskell-process-hayoo-ident ident)))
                (haskell-process-suggest-imports session file modules ident)))))
         ((string-match "^[ ]+It is a member of the hidden package [‘`‛]\\([^@\r\n]+\\).*['’].$" msg)
          (when haskell-process-suggest-add-package
@@ -266,67 +264,58 @@ list of modules where missed IDENT was found."
                      (get-buffer-create "*haskell-process-log*")
                      (get-buffer-create "*haskell-process-log*"))
       (switch-to-buffer-other-window (get-buffer "*haskell-process-log*")))
-     (t (haskell-process-queue-command
-         process
-         (make-haskell-command
-          :state (list (haskell-interactive-session) process command 0)
-          :go
-          (lambda (state)
-            (haskell-process-send-string
-             (cadr state)
-             (format haskell-process-do-cabal-format-string
-                     (haskell-session-cabal-dir (car state))
-                     (format "%s %s"
-                             (cl-ecase (haskell-process-type)
-                               ('ghci haskell-process-path-cabal)
-                               ('cabal-repl haskell-process-path-cabal)
-                               ('cabal-new-repl haskell-process-path-cabal)
-                               ('cabal-ghci haskell-process-path-cabal)
-                               ('stack-ghci haskell-process-path-stack))
-                             (cl-caddr state)))))
-          :live
-          (lambda (state buffer)
-            (let ((cmd (replace-regexp-in-string "^\\([a-z]+\\).*"
-                                                 "\\1"
-                                                 (cl-caddr state))))
-              (cond ((or (string= cmd "build")
-                         (string= cmd "install"))
-                     (haskell-process-live-build (cadr state) buffer t))
-                    (t
-                     (haskell-process-cabal-live state buffer)))))
-          :complete
-          (lambda (state response)
-            (let* ((process (cadr state))
-                   (session (haskell-process-session process))
-                   (message-count 0)
-                   (cursor (haskell-process-response-cursor process)))
-              ;; XXX: what the hell about the rampant code duplication?
-              (haskell-process-set-response-cursor process 0)
-              (while (haskell-process-errors-warnings nil session process response)
-                (setq message-count (1+ message-count)))
-              (haskell-process-set-response-cursor process cursor)
-              (let ((msg (format "Complete: cabal %s (%s compiler messages)"
-                                 (cl-caddr state)
-                                 message-count)))
-                (haskell-interactive-mode-echo session msg)
-                (when (= message-count 0)
-                  (haskell-interactive-mode-echo
-                   session
-                   "No compiler messages, dumping complete output:")
-                  (haskell-interactive-mode-echo session response))
-                (haskell-mode-message-line msg)
-                (when (and haskell-notify-p
-                           (fboundp 'notifications-notify))
-                  (notifications-notify
-                   :title (format "*%s*" (haskell-session-name (car state)))
-                   :body msg
-                   :app-name (cl-ecase (haskell-process-type)
-                               ('ghci haskell-process-path-cabal)
-                               ('cabal-repl haskell-process-path-cabal)
-                               ('cabal-new-repl haskell-process-path-cabal)
-                               ('cabal-ghci haskell-process-path-cabal)
-                               ('stack-ghci haskell-process-path-stack))
-                   :app-icon haskell-process-logo)))))))))))
+     (t (let ((app-name (cl-ecase (haskell-process-type)
+                          ('ghci haskell-process-path-cabal)
+                          ('cabal-repl haskell-process-path-cabal)
+                          ('stack-ghci haskell-process-path-stack))))
+          (haskell-process-queue-command
+           process
+           (make-haskell-command
+            :state (list (haskell-interactive-session) process command 0)
+            :go
+            (lambda (state)
+              (haskell-process-send-string
+               (cadr state)
+               (format haskell-process-do-cabal-format-string
+                       (haskell-session-cabal-dir (car state))
+                       (format "%s %s" app-name (cl-caddr state)))))
+            :live
+            (lambda (state buffer)
+              (let ((cmd (replace-regexp-in-string "^\\([a-z]+\\).*"
+                                                   "\\1"
+                                                   (cl-caddr state))))
+                (cond ((or (string= cmd "build")
+                           (string= cmd "install"))
+                       (haskell-process-live-build (cadr state) buffer t))
+                      (t
+                       (haskell-process-cabal-live state buffer)))))
+            :complete
+            (lambda (state response)
+              (let* ((process (cadr state))
+                     (session (haskell-process-session process))
+                     (message-count 0)
+                     (cursor (haskell-process-response-cursor process)))
+                (haskell-process-set-response-cursor process 0)
+                (while (haskell-process-errors-warnings nil session process response)
+                  (setq message-count (1+ message-count)))
+                (haskell-process-set-response-cursor process cursor)
+                (let ((msg (format "Complete: cabal %s (%s compiler messages)"
+                                   (cl-caddr state)
+                                   message-count)))
+                  (haskell-interactive-mode-echo session msg)
+                  (when (= message-count 0)
+                    (haskell-interactive-mode-echo
+                     session
+                     "No compiler messages, dumping complete output:")
+                    (haskell-interactive-mode-echo session response))
+                  (haskell-mode-message-line msg)
+                  (when (and haskell-notify-p
+                             (fboundp 'notifications-notify))
+                    (notifications-notify
+                     :title (format "*%s*" (haskell-session-name (car state)))
+                     :body msg
+                     :app-name app-name
+                     :app-icon haskell-process-logo))))))))))))
 
 (defun haskell-process-echo-load-message (process buffer echo-in-repl th)
   "Echo a load message."
