@@ -1,4 +1,4 @@
-(define-package "haskell-mode" "17.1" "A Haskell editing mode"
+(define-package "haskell-mode" "17.2-git" "A Haskell editing mode"
   '((emacs "25.1"))
   :url "https://github.com/haskell/haskell-mode"
   :keywords '("haskell" "cabal" "ghc" "repl"))
