@@ -1,4 +1,4 @@
-;;; haskell-move-nested.el --- Change the column of text nested below a line
+;;; haskell-move-nested.el --- Change the column of text nested below a line -*- lexical-binding: t -*-
 
 ;; Copyright (C) 2010  Chris Done
 
@@ -57,8 +57,7 @@ Use a numeric prefix argument to indicate amount of indentation to apply.
 In Transient Mark mode, if the mark is active, operate on the contents
 of the region instead."
   (interactive "p")
-  (haskell-move-nested cols)
-  )
+  (haskell-move-nested cols))
 
 ;;;###autoload
 (defun haskell-move-nested-left (cols)
@@ -69,8 +68,7 @@ Use a numeric prefix argument to indicate amount of indentation to apply.
 In Transient Mark mode, if the mark is active, operate on the contents
 of the region instead."
   (interactive "p")
-  (haskell-move-nested (- cols))
-  )
+  (haskell-move-nested (- cols)))
 
 (defun haskell-move-nested-region ()
   "Infer region off-side-rule block adjacent to point.
