@@ -471,6 +471,15 @@
    '(("Title" t default)
      ("Books" t default))))
 
+(ert-deftest haskell-syntactic-test-quasiquoter-sql-3 ()
+  "Embedded SQL statements"
+  (check-properties
+   '("sql = [Mod.sql| SELECT title FROM books; |]")
+   '(("SELECT" t font-lock-keyword-face)
+     ("title" t default)
+     ("FROM" t font-lock-keyword-face)
+     ("books" t default))))
+
 
 (ert-deftest haskell-syntactic-test-special-not-redefined ()
   "QuasiQuote should not conflict with TemplateHaskell"
@@ -544,7 +553,7 @@
      ("Comment2" t haskell-literate-comment-face)
      ("code3" t haskell-definition-face)
      ("Comment3" t haskell-literate-comment-face))
-   'literate-haskell-mode))
+   'haskell-literate-mode))
 
 (ert-deftest haskell-literate-bird-2 ()
   ;; Haskell Report requires empty line before bird code block. So it
@@ -566,7 +575,7 @@
      ("Comment2" t haskell-literate-comment-face)
      ("code3" t haskell-definition-face)
      ("Comment3" t haskell-literate-comment-face))
-   'literate-haskell-mode))
+   'haskell-literate-mode))
 
 (ert-deftest haskell-literate-latex-1 ()
   (check-properties
@@ -588,7 +597,7 @@
      ("Comment2" t haskell-literate-comment-face)
      ("code3" t haskell-definition-face)
      ("Comment3" t haskell-literate-comment-face))
-   'literate-haskell-mode))
+   'haskell-literate-mode))
 
 (ert-deftest haskell-literate-mixed-1 ()
   ;; Although Haskell Report does not advice mixing modes, it is a
@@ -611,7 +620,7 @@
      ("Comment2" t haskell-literate-comment-face)
      ("code3" t haskell-definition-face)
      ("Comment3" t haskell-literate-comment-face))
-   'literate-haskell-mode))
+   'haskell-literate-mode))
 
 (ert-deftest haskell-type-instance ()
   "Fontify \"instance\" after \"type\""
